//
//  Ring.frag
//  ExplorerGlobe
//
//  Created by Anton Holmberg on 4/12/16.
//  Copyright © 2016 Smule. All rights reserved.
//

#ifdef GL_ES
precision highp float;
#else
// define qualifiers to empty so code can compile without deleting qualifiers
#define highp
#define mediump
#define lowp
#endif

#define PI_2 1.5707963268

uniform vec3 u_ringColor;
uniform float u_ringAlpha;
uniform float u_ringFalloff;
uniform float u_linearPercent;

varying float v_edge;


void main()
{
    // use a steps function to split the ring such that the inner
    // part of ring has a linear gradient and the outer part of ring
    // has a curved falloff using sine wave
    float sineStep = step(u_linearPercent, v_edge);
    float linearStep = (1.0 - sineStep);
    
    // determine alpha for linear section
    float linearAlpha = (v_edge / u_linearPercent);
    linearAlpha = pow(linearAlpha, u_ringFalloff);
    
    // determine alpha for cosine section
    float sineValue = (1.0 - v_edge) / (1.0 - u_linearPercent);
    float sineAlpha = sin(sineValue * PI_2);
    
    // mix the alphas together and apply to final color
    float finalAlpha = (linearAlpha * linearStep) + (sineAlpha * sineStep);
    finalAlpha *= u_ringAlpha;
    gl_FragColor = vec4(u_ringColor * finalAlpha, finalAlpha);
}
