//
//  SmuleGlobe.vert
//  ExplorerGlobe
//
//  Created by Anton Holmberg on 3/20/16.
//  Copyright © 2016 Smule. All rights reserved.
//

#ifdef GL_ES
precision highp float;
#else
// define qualifiers to empty so code can compile without deleting qualifiers
#define highp
#define mediump
#define lowp
#endif

#define PI 3.1415926538

attribute vec3 a_position;
attribute vec2 a_texCoord;

uniform mat4 u_modelViewMatrix;
uniform mat4 u_projectionMatrix;
uniform mat4 u_modelViewProjectionMatrix;
uniform vec3 u_globeCenterWS;

uniform vec4 u_pulseColor;
uniform float u_pulseDisplacementAmount;
uniform float u_pulseDisplacementFalloff;
uniform float u_pulseYPositionWS;

// NOTE: these aren't labeled correctly... the ones marked WS for world space
// are actually in "eye space" (they are being transformed by the modelViewMatrix.
// world space would just be transformed by model matrix, which we don't use
// for the globe). I should fix up the naming, but then there's something weird
// going on with u_cameraPositionWS in the fragment shader (it should be in world
// space, but only seems to work with these "eye space" coordinates). I think this
// weirdness stems from the way we calculate view matrix in GlobeCamera. This
// doesn't really matter as long as everything is working, but I'm leaving this
// comment so people know there's something weird.
varying vec3 v_positionMS; // model space
varying vec3 v_positionWS; // world space
varying vec3 v_normalWS; // world space
varying vec2 v_texCoord;


float calculatePulseFactor()
{
    // NOTE: keep this method in sync with the equivalent method in
    // fragment shader AND in SmuleGlobe.cpp
    float pulseFactor = 1.0 - abs(u_pulseYPositionWS - a_position.y);
    pulseFactor = clamp(pulseFactor, 0.0, 1.0);
    pulseFactor = (cos(-PI + pulseFactor * PI) + 1.0) * 0.5;
    return pulseFactor;
}


void main()
{
    v_texCoord = a_texCoord;
    v_positionMS = a_position;
    v_positionWS = (u_modelViewMatrix * vec4(a_position, 1.0)).xyz;

    // calculate normal for this vertex in world space
    v_normalWS = normalize(u_globeCenterWS - v_positionWS);
    
    // displace the vertex slightly using pulse
    float pulseFactor = calculatePulseFactor();
    pulseFactor = pow(pulseFactor, u_pulseDisplacementFalloff);
    float pulseAmount = pulseFactor * u_pulseDisplacementAmount;
    vec3 finalPos = a_position + normalize(a_position) * pulseAmount;
    
    gl_Position = u_modelViewProjectionMatrix * vec4(finalPos, 1.0);
}
